; PROTECT.ASM -- RESIDENT WRITE PROTECT F0R HARD DISK
; ===================================================
CSEG		SEGMENT
		ASSUME CS:CSEG
		ORG	100H
;
START:		JMP	INITIALIZE
;
OLDINT13	DD	?		;ORIGINAL INTERRUPT 13 VECTOR
SWITCH		DB	0FH		;ON/OFF SWITCH FOR PROTECTION
;
; NEW INTERRUPT 13 (BIOS DISK I/O)
; --------------------------------
NEWINT13	PROC 	FAR
		CMP	AH, 03H		;SEE IF DOING DISK WRITE
		JZ	CHECKSTAT	;IF WRITING CHECK DRIVE NUMBER
;
		CMP	AH, 05H		;SEE IF DOING DISK FORMAT
		JZ	CHECKSTAT	;IF FORMATTING, CHECK DRIVE NO.
;
CONTINUE:	JMP	CS:[OLDINT13]	;ALLOW OPERATION TO PROCEED
;
CHECKSTAT:	CMP	SWITCH, 00H	;IS PROTECT SWITCH ON
		JNZ	CONTINUE	;IF NOT, PROCEED
		CMP	DL, 00H		;WAS FLOPPY 'A' SELECTED
		JZ	CONTINUE	;IF YES, PROCEED
		CMP	DL, 01H		;WAS FLOPPY 'B' SELECTED
		JZ	CONTINUE	;IF YES, PROCEED
;
ABORT:		MOV	AH, 03H		;SET WRITE PROTECT ERROR CODE
		STC			;SET FAILURE STATUS
		RET	2		;RETURN EXISTING FLAGS
;
NEWINT13	ENDP
;
; PROGRAM INSTALLATION
; BEGIN BY SEARCHING FOR AN EXISTING COPY OF CODE
; -----------------------------------------------

INITIALIZE:	MOV	DX, OFFSET NEWINT13  ;OFFSET TO BEGIN CODE SEARCH
		MOV	AX, CS		;DS:SI POINTS TO DESTINATION
		MOV	ES, AX		;ES:DI POINTS TO SOURCE
;
NEXTSEG:	DEC	AX		;SEARCH PREVIOUS SEGMENT
		MOV	DS, AX		;LOAD NEW SEGMENT TO SEARCH
		MOV	SI, DX		;POINT TO BEGINNING OF STRING
		MOV 	DI, DX		;POINT TO BEGINNING OF STRING
; 4 WORDS MUST MATCH TO CONFIRM COPY OF PROGRAM EXISTS
		MOV	CX, 0004H	;FOUR WORDS MUST MATCH
		CLD			;CLEAR DF FOR AUTOINCREMENT
		REPE	CMPSW
		JNZ	NOTFOUND	;IF NO MATCH, KEEP TRYING
; A LOCAL COPY OF THE PROGRAM MAY EXIST IN AN INPUT BUFFER
; ANY LOCAL COPIES MAY BE IDENTIFIED BY SWITCH SET TO 0FH
		CMP	DS:SWITCH, 0FH	;IS THIS AN INSTALLED COPY
		JNZ	TOGGLESW	;IF REAL COPY, TOGGLE SWITCH
;
NOTFOUND:	CMP	AX, 0001H	;STOP SEARCHING AT LOW MEMORY
		JNZ	NEXTSEG
;
; IF WE DROP OUT OF THE LOOP WITHOUT FINDING CODE THEN WE MUST INSTALL IT
;
		MOV	SWITCH, 00H	;SET SWITCH TO ON
		MOV	AX, 3513H	;SETUP TO GET OLD VECTOR
		INT 	21H
;
; PUT OLD VECTOR IN MEMORY
		MOV	WORD PTR CS:[OLDINT13],   BX
		MOV	WORD PTR CS:[OLDINT13+2], ES
;
		PUSH	CS
		POP	DS		;SET DS TO CS
;
; SEND MESSAGE, EXIT AND REMAIN RESIDENT
		MOV	DX, OFFSET PROTECT_ON
		MOV	AH, 09H		;PRINT STRING FUNCTION CALL
		INT	21H
;
; MAKE INTERRUPT 13 POINT TO THIS PROGRAM
		MOV	DX, OFFSET NEWINT13
		MOV	AX, 2513H	;SET NEW INTERRUPT 13
		INT 	21H
;
		MOV	DX, OFFSET INITIALIZE  ;NO. OF BYTES TO STAY
		INT	27H		;TERMINATE AND STAY RESIDENT
;
; IF PROGRAMS EXISTS IN MEMORY, THEN WE MUST ONLY TOGGLE THE SWITCH
TOGGLESW:	NOT	DS:SWITCH	;DS HAS BEEN SET BY SEARCH
		CMP	DS:SWITCH, 00H	;IS SWITCH ON
		JZ	ON
		MOV	DX, OFFSET PROTECT_OFF
		JMP	EXIT
ON:		MOV	DX, OFFSET PROTECT_ON
EXIT:		MOV	AH, 09H		;PRINT STRING FUNCTION CALL
		PUSH	CS
		POP	DS		;RESTORE DS
		INT	21H
		INT	20H		;EXIT TO DOS
;
PROTECT_ON	DB	"Hard Disk Protection On$"
PROTECT_OFF	DB	"Hard Disk Protection Off$"
CSEG		ENDS
;
		END 	START	
